require 'mustang'
require 'hpricot'

module Mike
  module Env
    def print(*args)
      puts(*args)
    end
  end # Env

  class Document
    attr_reader :js
    attr_reader :url
    attr_reader :dom
    attr_reader :html

    def initialize(url)
      @url  = url
      @html = load_url(@url)
      @dom  = Hpricot(@html)
      @js   = Mustang::Context.new

      prepare_context
      process_scripts
    end

    def scripts
      (@dom/"//script")
    end

    private

    def load_url(url)
      File.read(url) # just for the test
    end

    def prepare_context
      Env.instance_methods(false).each { |meth|
        @js.set :print, Env.instance_method(meth)
      }
    end

    def process_scripts
      scripts.each { |script|
        unless script.attributes["src"].empty?
          @js.load(script.attributes["src"])
        else
          @js.eval(script.html)
        end
      }
    end
  end # Document

  class Browser
    def initialize
    end

    def open(url)
      doc = Document.new(url)
      yield doc if block_given?
      doc
    end
  end # Browser
end # Mike
